# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import os
import subprocess
import io

import pandas as pd

from tqdm import tqdm
import concurrent.futures
import pymol2

from ..functions import *

pocket_bound_name = "Bound"
pocket_unbound_name = "Unbound"

pocket_info_dict = {
    "Druggability Score": "drug_score",
    "Volume": "pock_vol",
    "Number of Alpha Spheres": "nb_AS",
}


def run_dpocket(pharm_dict, chainid_dict=None, pocket_dir=None):

    pocket_dir_path = get_dir_path(dir_str=pocket_str, dir_path=pocket_dir)

    dp_file_path = get_file_path(
        pocket_table_file, dir_str=pocket_str, dir_path=pocket_dir
    )

    coord_path_dict = dict()
    run_chainid_dict = dict()

    with open(dp_file_path, "w") as file:
        for coord_path in list(pharm_dict.keys()):

            coord_name = get_file_name(coord_path)
            coord_name = coord_name.replace("_core", "")

            run_path = get_file_path(
                coord_name, dir_str=pocket_str, dir_path=pocket_dir
            )

            coord_path_dict[run_path] = coord_path
            if chainid_dict is not None:
                run_chainid_dict[run_path] = chainid_dict[coord_path]

            copy_path(coord_path, run_path)

            file.write(f"{run_path}\t{pharm_dict[coord_path]}\n")

    os.system(f"dpocket -f {dp_file_path} -o {pocket_dir_path}/pocket")

    fp_file_path = get_file_path("fp.txt", dir_str=pocket_str, dir_path=pocket_dir)
    fpn_file_path = get_file_path("fpn.txt", dir_str=pocket_str, dir_path=pocket_dir)
    exp_file_path = get_file_path("exp.txt", dir_str=pocket_str, dir_path=pocket_dir)

    df = pd.read_csv(
        fp_file_path,
        delim_whitespace=True,
        dtype=str,
    )

    pocket_dict = dict()

    for run_path in tqdm(
        lst_col(df, "pdb", unique=True),
        desc="Preparing pockets",
        position=0,
        leave=True,
    ):

        run_df = mask_equal(df, "pdb", run_path)

        if len(run_df) > 1:
            run_df = mask_equal(run_df, "overlap", run_df["overlap"].max())

        run_name = get_file_name(run_path)
        obj = run_name.replace(".pdb", "")

        run_dir_path = f"{pocket_dir_path}/{obj}_out"
        run_pockets_dir_path = f"{run_dir_path}/pockets"

        info_file_path = get_file_path(f"{obj}_info.txt", dir_path=run_dir_path)

        add_pocket = False
        with open(info_file_path, "r") as file:
            for line in file.readlines():
                if line[:6] == "Pocket":
                    pocket_id = line[7:8]
                    count = 0
                for info, col in pocket_info_dict.items():
                    info_str = f"{info} :"
                    if info_str in line:
                        info_val = round(
                            float(line.split(info_str)[1].replace(" ", ""))
                        )
                        col_val = round(float(run_df.at[0, col]))
                        if col_val == info_val:
                            count += 1
                if count == 3:
                    add_pocket = True
                    break

        if add_pocket:
            cont_path = get_file_path(
                f"pocket{pocket_id}_atm.pdb", dir_path=run_pockets_dir_path
            )
            structure = load_coord(cont_path)

            pocket_residue_lst = get_residues(structure)

            if chainid_dict is not None:
                chainid_lst = run_chainid_dict[run_path]
                pocket_residue_lst = [
                    x for x in pocket_residue_lst if get_reschainid(x) in chainid_lst
                ]

            pocket_cont_lst = sort_lst(
                lst_unique(
                    [
                        get_resnum(x)
                        for x in pocket_residue_lst
                        if (get_resnum(x) < 50000) and is_aa(x)
                    ]
                )
            )

            pocket_lig = run_df.at[0, "lig"]
            pocket_path = get_pocket_path(
                obj.replace("pocket_", ""),
                pocket_lig,
                dir_path=pocket_dir,
            )

            coord_path = coord_path_dict[run_path]

            copy_path(coord_path, pocket_path)

            pocket_dict[coord_path] = {pocket_lig: dict()}

            pocket_dict[coord_path][pocket_lig][pocket_volume_col] = run_df.at[
                0, "pock_vol"
            ]
            pocket_dict[coord_path][pocket_lig][pocket_score_col] = run_df.at[
                0, "drug_score"
            ]
            pocket_dict[coord_path][pocket_lig][pocket_status_col] = pocket_bound_name
            pocket_dict[coord_path][pocket_lig][pocket_type_col] = title_str(
                pharm_lig_col
            )
            pocket_dict[coord_path][pocket_lig][pocket_lig_col] = pocket_lig
            pocket_dict[coord_path][pocket_lig][pocket_cont_col] = lst_to_str(
                pocket_cont_lst
            )
            pocket_dict[coord_path][pocket_lig][pocket_path_col] = pocket_path

        delete_path(run_path)
        delete_path(run_dir_path)

    delete_path(dp_file_path)
    delete_path(fp_file_path)
    delete_path(fpn_file_path)
    delete_path(exp_file_path)

    return pocket_dict


def run_fpocket(
    coord_path,
    pocket_dir=None,
    chainid_lst=None,
    check_lig=False,
    check_dist=3.5,
    check_min_simi=0.1,
    use_simpson=True,
):

    pocket_dir_path = get_dir_path(dir_str=pocket_str, dir_path=pocket_dir)

    coord_name = get_file_name(coord_path)
    coord_name = coord_name.replace("_core", "")

    run_path = get_file_path(coord_name, dir_str=pocket_str, dir_path=pocket_dir)

    copy_path(coord_path, run_path)

    cmd_lst = [
        "fpocket",
        "-f",
        run_path,
        "-d",
    ]

    df = pd.read_csv(
        io.StringIO(
            subprocess.Popen(cmd_lst, stdout=subprocess.PIPE)
            .communicate()[0]
            .decode("utf-8")
        ),
        delim_whitespace=True,
        dtype=str,
    )

    if chainid_lst is not None:
        for chainid in chainid_lst:
            df = mask_equal(df, "name_chain_1", chainid)
            df = mask_equal(df, "name_chain_2", chainid)

    pocket_dict = {coord_path: dict()}

    obj = coord_name.replace(".pdb", "")

    run_dir_path = f"{pocket_dir_path}/pocket_{obj}_out"

    with pymol2.PyMOL() as pymol:

        cmd = pymol.cmd

        cmd.load(coord_path, obj)

        for i, index in enumerate(list(df.index.values)):

            pocket = i + 1

            pocket_id = df.at[index, "cav_id"]

            pocket_chainid_lst = lst_unique(
                (df.at[index, "name_chain_1"], df.at[index, "name_chain_2"])
            )

            if check_lig:
                pocket_chainid_sele = lst_to_str(pocket_chainid_lst, join_txt="+")

            pocket_chainid = lst_to_str(
                pocket_chainid_lst,
                join_txt="_",
            )

            cont_path = get_file_path(
                f"pocket{pocket_id}_atm.pdb", dir_path=run_dir_path
            )
            sphere_path = get_file_path(
                f"pocket{pocket_id}_vert.pqr", dir_path=run_dir_path
            )

            sphere_obj = f"sphere_{pocket}"
            cmd.load(sphere_path, sphere_obj)

            cmd.alter(sphere_obj, f"chain='{pocket_chainid}'")

            pocket_obj = f"pocket_{pocket}"
            cmd.create(pocket_obj, f"{obj} {sphere_obj}")

            pocket_path = get_pocket_path(
                obj,
                pocket,
                dir_path=pocket_dir,
            )
            cmd.save(pocket_path, pocket_obj)

            structure = load_coord(cont_path)

            pocket_residue_lst = get_residues(structure)

            if chainid_lst is not None:
                pocket_residue_lst = [
                    x for x in pocket_residue_lst if get_reschainid(x) in chainid_lst
                ]

            pocket_cont_lst = sort_lst(
                lst_unique(
                    [
                        get_resnum(x)
                        for x in pocket_residue_lst
                        if (get_resnum(x) < 50000) and is_aa(x)
                    ]
                )
            )

            pocket_lig = df.at[index, "lig_het_tag"]

            pocket_status = pocket_unbound_name
            pocket_type = pocket_unbound_name
            if pd.isna(pocket_lig):
                pocket_lig = "STP"
            else:
                type_pocket = True

                if check_lig:
                    lig_cont_str = f"byres polymer and chain {pocket_chainid_sele} within {check_dist} of resname {pocket_lig}"
                    lig_cont_dict = {
                        "lig_cont_lst": list(),
                    }

                    cmd.iterate(
                        lig_cont_str,
                        "lig_cont_lst.append(resi)",
                        space=lig_cont_dict,
                    )

                    lig_cont_lst = [
                        int(x) for x in lst_unique(lig_cont_dict["lig_cont_lst"])
                    ]

                    if use_simpson:
                        check_simi = calc_simpson(pocket_cont_lst, lig_cont_lst)
                    else:
                        check_simi = calc_jaccard(pocket_cont_lst, lig_cont_lst)

                    if check_simi <= check_min_simi:
                        pocket_lig = "STP"
                        type_pocket = False

                if type_pocket:
                    pocket_status = pocket_bound_name
                    is_pharm = True
                    for lig_col in lig_col_lst:
                        if pocket_lig in lig_lst_dict[lig_col]:
                            pocket_type = lig_col
                            is_pharm = False
                    if is_pharm:
                        pocket_type = pharm_lig_col
                    pocket_type = title_str(pocket_type)

            pocket_dict[coord_path][pocket] = dict()

            pocket_dict[coord_path][pocket][pocket_volume_col] = df.at[index, "volume"]
            pocket_dict[coord_path][pocket][pocket_score_col] = df.at[
                index, "drug_score"
            ]
            pocket_dict[coord_path][pocket][pocket_status_col] = pocket_status
            pocket_dict[coord_path][pocket][pocket_type_col] = pocket_type
            pocket_dict[coord_path][pocket][pocket_lig_col] = pocket_lig
            pocket_dict[coord_path][pocket][pocket_cont_col] = lst_to_str(
                pocket_cont_lst
            )
            pocket_dict[coord_path][pocket][pocket_path_col] = pocket_path

    delete_path(run_path)
    delete_path(run_dir_path)

    return pocket_dict


def prep_pocket(
    coord_paths,
    pocket_dir=None,
    pocket_json_path=None,
    pharm_dict=None,
    chainid_dict=None,
    check_lig=False,
    check_dist=3.5,
    check_min_simi=0.1,
    use_simpson=True,
    update_pocket=False,
    num_cpu=1,
):

    pocket_dir_path = get_dir_path(dir_str=pocket_str, dir_path=pocket_dir)

    if update_pocket:
        delete_path(pocket_dir_path)
    append_path(pocket_dir_path)

    coord_path_lst = type_lst(coord_paths)

    pocket_dict = dict()

    if pharm_dict is not None:
        coord_path_lst = [x for x in coord_path_lst if x not in list(pharm_dict.keys())]
        pocket_dict = merge_dicts(
            [
                pocket_dict,
                run_dpocket(
                    pharm_dict,
                    chainid_dict=chainid_dict,
                    pocket_dir=pocket_dir,
                ),
            ]
        )

    if num_cpu == 1:
        for coord_path in tqdm(
            coord_path_lst,
            desc="Preparing pockets",
            position=0,
            leave=True,
        ):
            pocket_dict = merge_dicts(
                [
                    pocket_dict,
                    run_fpocket(
                        coord_path,
                        pocket_dir=pocket_dir,
                        chainid_lst=chainid_dict[coord_path],
                        check_lig=check_lig,
                        check_dist=check_dist,
                        check_min_simi=check_min_simi,
                        use_simpson=use_simpson,
                    ),
                ]
            )
    else:
        with concurrent.futures.ProcessPoolExecutor(max_workers=num_cpu) as executor:
            job_lst = [
                executor.submit(
                    run_fpocket,
                    coord_path,
                    pocket_dir=pocket_dir,
                    chainid_lst=chainid_dict[coord_path],
                    check_lig=check_lig,
                    check_dist=check_dist,
                    check_min_simi=check_min_simi,
                    use_simpson=use_simpson,
                )
                for coord_path in coord_path_lst
            ]

            for job in tqdm(
                concurrent.futures.as_completed(job_lst),
                desc="Preparing pockets",
                total=len(job_lst),
                miniters=1,
                position=0,
                leave=True,
            ):

                pocket_dict = merge_dicts([pocket_dict, job.result()])

    print("Prepared pockets!")

    if pocket_json_path is not None:
        save_json(pocket_json_path, pocket_dict)
    else:
        return pocket_dict


# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import os
import pandas as pd
import xml.etree.ElementTree as ET
import gzip
from tqdm import tqdm
import concurrent.futures
from Bio.PDB import Select
import pymol2

from ..functions import *

max_cb_dist = 12.0
max_atomid_dist = 5.0


class ChainSelect(Select):
    def __init__(self, sele_dict):
        self.sele_dict = sele_dict

    def accept_residue(self, residue):

        return self.sele_dict[get_reschainid(residue)][get_resid(residue)]

    def accept_atom(self, atom):

        if get_atomid(atom)[0] == "H":
            return 0
        else:
            return 1


def run_pdb_renum(
    pdb_code_lst,
    renum_script_path,
    rcsb_dir=None,
    sifts_dir=None,
    renum_dir=None,
    num_cpu=1,
):

    renum_pdb_code_lst = [
        x
        for x in pdb_code_lst
        if not path_exists(get_renum_path(x, dir_path=renum_dir))
    ]

    if len(renum_pdb_code_lst) > 0:
        pdb_code_str = lst_to_str(
            renum_pdb_code_lst,
            join_txt=" ",
        )

        cmd_lst = [
            f"python {renum_script_path}",
            f"-rfla {pdb_code_str}",
            "-mmCIF",
            f"-sipm {get_dir_path(dir_str=rcsb_str, dir_path=rcsb_dir)}",
            f"-sips {get_dir_path(dir_str=sifts_str,dir_path=sifts_dir)}",
            f"-sopm {get_dir_path(dir_str=renum_str, dir_path=renum_dir)}",
            "-offz",
            f"-nproc {num_cpu}",
        ]

        cmd_str = lst_to_str(cmd_lst, join_txt=" ")

        os.system(cmd_str)

        assembly_cmd_lst = [
            f"python {renum_script_path}",
            f"-rfla {pdb_code_str}",
            "-mmCIF_assembly",
            f"-sipma {get_dir_path(dir_str=rcsb_assembly_str, dir_path=rcsb_dir)}",
            f"-sips {get_dir_path(dir_str=sifts_str,dir_path=sifts_dir)}",
            f"-sopma {get_dir_path(dir_str=renum_assembly_str, dir_path=renum_dir)}",
            "-offz",
            f"-nproc {num_cpu}",
        ]

        assembly_cmd_str = lst_to_str(assembly_cmd_lst, join_txt=" ")

        os.system(assembly_cmd_str)


def get_sifts_dict(sifts_path):

    tree = ET.parse(gzip.open(sifts_path, "rt"))
    root = tree.getroot()
    base = "{http://www.ebi.ac.uk/pdbe/docs/sifts/eFamily.xsd}"

    sifts_dict = dict()
    uniprot_dict = dict()

    pdb_chainid_lst = list()
    uniprot_chainid_lst = list()

    for entity in root:
        if entity.tag == (str(base) + "entity"):
            for segment in entity:
                if segment.tag == (str(base) + "segment"):
                    for listResidue in segment:
                        if listResidue.tag == (str(base) + "listResidue"):
                            for residue in listResidue:
                                if residue.tag == (str(base) + "residue"):
                                    if residue.get("dbSource") == "PDBe":

                                        pdbe_resid = residue.get("dbResNum")

                                        pdb_resid = "null"
                                        uniprot_resid = "null"

                                        no_uniprot = True

                                        for crossRefDb in residue:
                                            if crossRefDb.get("dbSource") == "PDB":
                                                pdb_code = crossRefDb.get(
                                                    "dbAccessionId"
                                                )
                                                chainid = crossRefDb.get("dbChainId")

                                                if chainid not in pdb_chainid_lst:
                                                    pdb_chainid_lst.append(chainid)

                                                pdb_resid = crossRefDb.get("dbResNum")

                                            if crossRefDb.get("dbSource") == "UniProt":

                                                if chainid not in uniprot_chainid_lst:
                                                    uniprot_chainid_lst.append(chainid)

                                                no_uniprot = False
                                                uniprot_resid = crossRefDb.get(
                                                    "dbResNum"
                                                )

                                        if no_uniprot:
                                            uniprot_resid = str(
                                                get_str_num(pdbe_resid) + 50000
                                            )

                                            if chainid not in list(uniprot_dict.keys()):
                                                uniprot_dict[chainid] = dict()

                                            uniprot_dict[chainid][
                                                uniprot_resid
                                            ] = pdb_resid

                                        if (
                                            pdb_resid != "null"
                                            and uniprot_resid != "null"
                                        ):
                                            if pdb_code not in list(sifts_dict.keys()):
                                                sifts_dict[pdb_code] = dict()

                                            if chainid not in list(
                                                sifts_dict[pdb_code].keys()
                                            ):
                                                sifts_dict[pdb_code][chainid] = dict()

                                            sifts_dict[pdb_code][chainid][
                                                pdb_resid
                                            ] = uniprot_resid

    for chainid in pdb_chainid_lst:
        if chainid not in uniprot_chainid_lst:
            if chainid in list(uniprot_dict.keys()):
                for pdb_resid in list(sifts_dict[pdb_code][chainid].keys()):
                    uniprot_resid = sifts_dict[pdb_code][chainid][pdb_resid]
                    if int(uniprot_resid) > 50000:
                        sifts_dict[pdb_code][chainid][pdb_resid] = uniprot_dict[
                            chainid
                        ][uniprot_resid]

    return sifts_dict


def build_sifts_map(sifts_path_lst, sifts_json_path, num_cpu=1):

    sifts_dict = {}

    with concurrent.futures.ProcessPoolExecutor(max_workers=num_cpu) as executor:

        job_lst = [
            executor.submit(get_sifts_dict, sifts_path) for sifts_path in sifts_path_lst
        ]

        for job in tqdm(
            concurrent.futures.as_completed(job_lst),
            desc="Preparing SIFTS numbering",
            total=len(job_lst),
            miniters=1,
            position=0,
            leave=True,
        ):

            sifts_dict = merge_dicts([sifts_dict, job.result()])

    save_json(sifts_json_path, sifts_dict)


def build_sele_dict(
    structure,
    curr_chainid,
    chainid_lst,
    max_lig_dist=4,
    min_lig_cont=5,
    prot_resid_lst=None,
    lig_resid_lst=None,
    bound_lig_lst=None,
    bound_chainid_lst=None,
):

    sele_dict = dict()
    sele_df = pd.DataFrame()

    for model in structure:

        modelid = get_modelid(model)

        sele_dict[modelid] = dict()

        curr_chain = model[curr_chainid]

        neighbors = get_neighbors(curr_chain)

        resnum_lst = list()
        bound_residue_dict = dict()
        bound_chain_dict = dict()

        for chain in model:

            chainid = get_chainid(chain)

            sele_dict[modelid][chainid] = dict()

            if chainid not in chainid_lst:

                sele = 0

                check_chainid = True
                if bound_chainid_lst is not None:
                    if chainid not in bound_chainid_lst:
                        check_chainid = False

                if check_chainid:

                    atomid_chain_cont = [
                        x
                        for x in get_chain_cont(
                            neighbors,
                            chain,
                            max_dist=max_atomid_dist,
                        )
                        if is_aa(x.get_parent())
                        and get_reschainid(x.get_parent()) == curr_chainid
                        and get_resnum(x.get_parent()) < 50000
                    ]

                    cb_chain_cont = [
                        x
                        for x in get_chain_cont(
                            neighbors,
                            chain,
                            max_dist=max_cb_dist,
                        )
                        if is_aa(x.get_parent())
                        and get_reschainid(x.get_parent()) == curr_chainid
                        and get_resnum(x.get_parent()) < 50000
                        and get_atomid(x) == "CB"
                    ]

                    if prot_resid_lst is not None:
                        atomid_chain_cont = [
                            x
                            for x in atomid_chain_cont
                            if get_resnum(x.get_parent()) in prot_resid_lst
                        ]
                        cb_chain_cont = [
                            x
                            for x in cb_chain_cont
                            if get_resnum(x.get_parent()) in prot_resid_lst
                        ]

                    atomid_chain_cont = len(atomid_chain_cont)
                    cb_chain_cont = len(cb_chain_cont)

                    chain_seq = join_seq_lst(get_seq_lst(chain))

                    if (
                        cb_chain_cont >= 10 and atomid_chain_cont >= 1
                    ) or atomid_chain_cont >= 5:

                        if chain_seq not in list(bound_chain_dict.keys()):
                            bound_chain_dict[chain_seq] = dict()
                        bound_chain_dict[chain_seq][chainid] = cb_chain_cont

                for residue in chain:
                    resid = get_resid(residue)
                    sele_dict[modelid][chainid][resid] = sele
            else:
                for residue in chain:

                    resid = get_resid(residue)
                    sele = 0

                    if is_aa(residue):

                        if chainid == curr_chainid:
                            resnum = get_resnum(residue)
                            if resnum < 50000:
                                resnum_lst.append(resnum)
                            sele = 1

                    elif is_het(residue):

                        atomid_residue_cont = [
                            x
                            for x in get_residue_cont(
                                neighbors,
                                residue,
                                max_dist=max_lig_dist,
                            )
                            if (
                                is_het(x.get_parent())
                                and get_reschainid(x.get_parent()) == curr_chainid
                            )
                            or (
                                is_aa(x.get_parent())
                                and get_reschainid(x.get_parent()) == curr_chainid
                                and get_resnum(x.get_parent()) < 50000
                            )
                        ]

                        if lig_resid_lst is not None:
                            atomid_residue_cont = [
                                x
                                for x in atomid_residue_cont
                                if is_het(x.get_parent())
                                or (
                                    is_aa(x.get_parent())
                                    and get_resnum(x.get_parent()) in lig_resid_lst
                                )
                            ]

                        atomid_residue_cont = len(atomid_residue_cont)

                        if atomid_residue_cont >= min_lig_cont:
                            resname = get_resname(residue)
                            if resname not in list(bound_residue_dict.keys()):
                                bound_residue_dict[resname] = dict()
                            bound_residue_dict[resname][
                                (chainid, resid)
                            ] = atomid_residue_cont

                    elif is_wat(residue):
                        if chainid == curr_chainid:
                            sele = 1

                    sele_dict[modelid][chainid][resid] = sele

        bound_prot_chainid_lst = list()
        for chain_seq in list(bound_chain_dict.keys()):
            bound_chain_lst = list(bound_chain_dict[chain_seq].keys())
            bound_chain_cont_lst = [
                bound_chain_dict[chain_seq][x] for x in bound_chain_lst
            ]
            bound_chain = bound_chain_lst[
                bound_chain_cont_lst.index(max(bound_chain_cont_lst))
            ]
            bound_prot_chainid_lst.append(bound_chain)
            for residue in model[bound_chain]:
                resid = get_resid(residue)
                sele_dict[modelid][bound_chain][resid] = 1

        bound_lig_lst = list()
        for resname in list(bound_residue_dict.keys()):
            bound_residue_lst = list(bound_residue_dict[resname].keys())
            bound_residue_cont_lst = [
                bound_residue_dict[resname][x] for x in bound_residue_lst
            ]
            bound_residue = bound_residue_lst[
                bound_residue_cont_lst.index(max(bound_residue_cont_lst))
            ]
            bound_lig_lst.append(resname)
            sele_dict[modelid][bound_residue[0]][bound_residue[1]] = 1

        sele_df.at[modelid, modelid_col] = modelid
        sele_df.at[modelid, bound_lig_col] = lst_to_str(bound_lig_lst)
        sele_df.at[modelid, bound_prot_chainid_col] = lst_to_str(bound_prot_chainid_lst)
        sele_df.at[modelid, range_col] = res_to_str(resnum_lst)

    return sele_dict, sele_df


def isolate_chains(
    pdb_code,
    pdb_dict,
    core_dir=None,
    rcsb_dir=None,
    sifts_dir=None,
    renum_dir=None,
    max_lig_dist=4,
    min_lig_cont=5,
    prot_resid_lst=None,
    lig_resid_lst=None,
    add_h=False,
    add_h_his=False,
    bound_chainid_dict=None,
    all_models=False,
    update_coords=False,
):

    rcsb_path = get_rcsb_path(pdb_code, dir_path=rcsb_dir)
    renum_path = get_renum_path(pdb_code, dir_path=renum_dir)
    sifts_path = get_sifts_path(pdb_code, dir_path=sifts_dir)

    chainid_lst = pdb_dict[pdb_code]

    structure = load_coord(renum_path)

    if not all_models:
        structure = type_lst(structure[0])

    chain_df = pd.DataFrame()

    with pymol2.PyMOL() as pymol:
        cmd = pymol.cmd

        for chainid in chainid_lst:

            bound_chainid_lst = None
            if bound_chainid_dict is not None:
                pdb_id = f"{pdb_code}{chainid}"
                if pdb_id in (bound_chainid_dict.keys()):
                    bound_chainid_lst = bound_chainid_dict[pdb_id]

            rcsb_assembly_path = "None"
            renum_assembly_path = "None"

            if bound_chainid_lst is None:

                renum_assembly_dir = get_dir_path(
                    dir_str=renum_assembly_str, dir_path=renum_dir
                )
                rcsb_assembly_dir = get_dir_path(
                    dir_str=rcsb_assembly_str, dir_path=rcsb_dir
                )

                for renum_assembly_file in search_dir(renum_assembly_dir, pdb_code):

                    renum_assembly_path = f"{renum_assembly_dir}/{renum_assembly_file}"
                    rcsb_assembly_path = f"{rcsb_assembly_dir}/{renum_assembly_file.replace('_renum.cif','.cif.gz')}"

                    assembly_dict = load_cif_dict(renum_assembly_path)

                    assembly_chainid_lst = lst_unique(
                        assembly_dict["_pdbe_chain_remapping.orig_auth_asym_id"]
                    )

                    if chainid in assembly_chainid_lst:
                        if (
                            len(
                                [
                                    x
                                    for x in assembly_chainid_lst
                                    if x in chainid_lst and x != chainid
                                ]
                            )
                            == 0
                        ):
                            bound_chainid_lst = [
                                x for x in assembly_chainid_lst if x != chainid
                            ]

            sele_dict, sele_df = build_sele_dict(
                structure,
                chainid,
                chainid_lst,
                max_lig_dist=max_lig_dist,
                min_lig_cont=min_lig_cont,
                prot_resid_lst=prot_resid_lst,
                lig_resid_lst=lig_resid_lst,
                bound_chainid_lst=bound_chainid_lst,
            )

            sele_df[pdb_code_col] = pdb_code
            sele_df[chainid_col] = chainid
            sele_df[rcsb_path_col] = rcsb_path
            sele_df[renum_path_col] = renum_path
            sele_df[sifts_path_col] = sifts_path
            sele_df[rcsb_assembly_path_col] = rcsb_assembly_path
            sele_df[renum_assembly_path_col] = renum_assembly_path

            for modelid in list(sele_dict.keys()):

                modelid_str = modelid
                if not all_models:
                    modelid_str = None

                cif_path = get_core_path(
                    pdb_code, chainid, modelid=modelid_str, dir_path=core_dir
                )
                pdb_path = get_core_path(
                    pdb_code,
                    chainid,
                    modelid=modelid_str,
                    return_pdb=True,
                    dir_path=core_dir,
                )

                update_cif = True
                if not update_coords:
                    if path_exists(cif_path):
                        update_cif = False

                update_pdb = True
                if not update_coords:
                    if path_exists(cif_path):
                        update_pdb = False

                sele_df.at[int(modelid), core_path_col] = cif_path

                if update_cif:
                    chain_sele = ChainSelect(sele_dict[modelid])
                    save_coord(cif_path, structure[int(modelid)], sele=chain_sele)

                if update_pdb:
                    pymol_obj = f"{pdb_code}{chainid}"

                    if all_models:
                        pymol_obj += str(modelid)

                    cmd.load(cif_path, pymol_obj)
                    cmd.save(pdb_path, pymol_obj)

                if add_h:
                    pdb_h_path = get_core_path(
                        pdb_code,
                        chainid,
                        modelid=modelid_str,
                        return_pdb=True,
                        add_h=True,
                        dir_path=core_dir,
                    )

                    update_pdb_h = True
                    if not update_coords:
                        if path_exists(pdb_h_path):
                            update_pdb_h = False

                    if update_pdb_h:
                        cmd_lst = ["reduce", pdb_path, ">", pdb_h_path, "-NOHETh"]

                        if add_h_his:
                            cmd_lst.append("-HIS")

                        os.system(lst_to_str(cmd_lst, join_txt=" "))

            sele_df[pdb_id_col] = sele_df[pdb_code_col].map(str) + sele_df[
                chainid_col
            ].map(str)

            if all_models:
                sele_df[pdb_id_col] += sele_df[modelid_col].map(str)

            sele_df[modelid_col] = 0

            chain_df = pd.concat([chain_df, sele_df], sort=False)

    return chain_df


def run_pdb_chain(
    pdb_ids,
    core_dir=None,
    rcsb_dir=None,
    sifts_dir=None,
    renum_dir=None,
    max_lig_dist=4,
    min_lig_cont=5,
    prot_resids=None,
    lig_resids=None,
    add_h=False,
    add_h_his=False,
    bound_chainid_dict=None,
    all_models=False,
    update_coords=False,
    num_cpu=1,
):

    append_path(get_dir_path(dir_str=core_str, dir_path=core_dir))

    pdb_id_lst = type_lst(pdb_ids)

    pdb_dict = build_pdb_dict(pdb_id_lst)

    prot_resid_lst = res_to_lst(prot_resids)
    lig_resid_lst = res_to_lst(lig_resids)

    df = pd.DataFrame()

    if num_cpu == 1:
        for pdb_code in tqdm(
            list(pdb_dict.keys()),
            desc="Isolating PDB chains",
            position=0,
            leave=True,
        ):
            df = pd.concat(
                [
                    df,
                    isolate_chains(
                        pdb_code,
                        pdb_dict,
                        core_dir=core_dir,
                        rcsb_dir=rcsb_dir,
                        sifts_dir=sifts_dir,
                        renum_dir=renum_dir,
                        max_lig_dist=max_lig_dist,
                        min_lig_cont=min_lig_cont,
                        prot_resid_lst=prot_resid_lst,
                        lig_resid_lst=lig_resid_lst,
                        add_h=add_h,
                        add_h_his=add_h_his,
                        bound_chainid_dict=bound_chainid_dict,
                        all_models=all_models,
                        update_coords=update_coords,
                    ),
                ],
                sort=False,
            )
    else:
        with concurrent.futures.ProcessPoolExecutor(max_workers=num_cpu) as executor:
            job_lst = [
                executor.submit(
                    isolate_chains,
                    pdb_code,
                    pdb_dict,
                    core_dir=core_dir,
                    rcsb_dir=rcsb_dir,
                    sifts_dir=sifts_dir,
                    renum_dir=renum_dir,
                    max_lig_dist=max_lig_dist,
                    min_lig_cont=min_lig_cont,
                    prot_resid_lst=prot_resid_lst,
                    lig_resid_lst=lig_resid_lst,
                    add_h=add_h,
                    add_h_his=add_h_his,
                    bound_chainid_dict=bound_chainid_dict,
                    all_models=all_models,
                    update_coords=update_coords,
                )
                for pdb_code in list(pdb_dict.keys())
            ]

            for job in tqdm(
                concurrent.futures.as_completed(job_lst),
                desc="Isolating PDB chains",
                total=len(job_lst),
                miniters=1,
                position=0,
                leave=True,
            ):

                df = pd.concat([df, job.result()], sort=False)

    df = df.reset_index(drop=True)

    return df


def prep_coord(
    pdb_id_lst,
    renum_script_path,
    coord_table_path,
    core_dir=None,
    rcsb_dir=None,
    sifts_dir=None,
    renum_dir=None,
    sifts_json_path=None,
    max_lig_dist=4,
    min_lig_cont=5,
    prot_resids=None,
    lig_resids=None,
    add_h=False,
    add_h_his=False,
    bound_chainid_dict=None,
    all_models=False,
    data=None,
    update_coords=False,
    num_cpu=1,
):
    pdb_code_lst = build_pdb_code_lst(pdb_id_lst)

    run_pdb_renum(
        pdb_code_lst,
        renum_script_path,
        rcsb_dir=rcsb_dir,
        sifts_dir=sifts_dir,
        renum_dir=renum_dir,
        num_cpu=num_cpu,
    )

    delete_path("log_corrected.txt")
    delete_path("log_translator.txt")

    sifts_path_lst = [
        get_sifts_path(pdb_code, dir_path=sifts_dir) for pdb_code in pdb_code_lst
    ]

    build_sifts_map(sifts_path_lst, sifts_json_path, num_cpu=num_cpu)

    df = run_pdb_chain(
        pdb_id_lst,
        core_dir=core_dir,
        rcsb_dir=rcsb_dir,
        sifts_dir=sifts_dir,
        renum_dir=renum_dir,
        max_lig_dist=max_lig_dist,
        min_lig_cont=min_lig_cont,
        prot_resids=prot_resids,
        lig_resids=lig_resids,
        add_h=add_h,
        add_h_his=add_h_his,
        bound_chainid_dict=bound_chainid_dict,
        all_models=all_models,
        update_coords=update_coords,
        num_cpu=num_cpu,
    )

    if data is not None:
        df_col_lst = list(data.columns)

        for col in [pdb_id_col, bound_lig_col, bound_prot_chainid_col]:
            if col in df_col_lst:
                del data[col]

        df = merge_tables(df, data)

    save_table(coord_table_path, df)

    print("Prepared coordinate files!")



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import pandas as pd
import numpy as np

import statsmodels.api as sm
from statsmodels.sandbox.stats.multicomp import multipletests
from scipy import stats

from .lst import type_lst
from .table import lst_col, mask_equal, mask_unequal
from .col import (
    total_col,
    index_col,
    p_col,
    correct_p_col,
    corr_col,
    a_col,
    b_col,
    c_col,
    d_col,
    risk_ratio_col,
    low_ci_col,
    up_ci_col,
    sig_col,
)


def label_sig(val, label="*", cutoff=0.05):

    if val > cutoff:
        return "ns"
    else:
        return label


def correct_p_vals(df, correct_method="fdr_bh"):

    df = df.reset_index(drop=True)

    p_lst = lst_col(df, p_col)

    correct_p_lst = multipletests(p_lst, method=correct_method)[1]

    for index in list(df.index.values):

        correct_p = correct_p_lst[index]

        df.at[index, correct_p_col] = correct_p

    return df


def calc_rr(df, exp_cols, out_cols, correct_method="fdr_bh"):

    df = df.dropna()
    df = df.reset_index(drop=True)

    i_col_lst = type_lst(out_cols)
    j_col_lst = type_lst(exp_cols)

    col_lst = i_col_lst + j_col_lst

    if total_col not in list(df.columns):

        df.index.name = index_col
        df = df.reset_index()

        df = df.groupby(col_lst)[index_col].nunique().reset_index(name=total_col)

    total = df[total_col].sum()

    df_index_lst = list(df.index.values)

    for index in df_index_lst:

        i_df = df.copy(deep=True)
        j_df = df.copy(deep=True)

        for col in i_col_lst:

            val = df.at[index, col]

            i_df = mask_equal(i_df, col, val)

        for col in j_col_lst:

            val = df.at[index, col]

            j_df = mask_equal(j_df, col, val)

        a = df.at[index, total_col]
        b = i_df[total_col].sum() - a
        c = j_df[total_col].sum() - a
        d = total - (a + b + c)

        matrix = np.zeros((2, 2))

        matrix[0, 0] = a
        matrix[0, 1] = b
        matrix[1, 0] = c
        matrix[1, 1] = d

        table = sm.stats.Table2x2(matrix)

        risk_ratio = table.riskratio
        p_val = table.riskratio_pvalue()
        low_ci = table.riskratio_confint()[0]
        up_ci = table.riskratio_confint()[1]

        df.at[index, a_col] = a
        df.at[index, b_col] = b
        df.at[index, c_col] = c
        df.at[index, d_col] = d
        df.at[index, risk_ratio_col] = risk_ratio
        df.at[index, p_col] = p_val
        df.at[index, low_ci_col] = low_ci
        df.at[index, up_ci_col] = up_ci

    if correct_method is not None:
        df = correct_p_vals(df, correct_method=correct_method)

    df = df.sort_values(by=col_lst)

    noise_df = pd.DataFrame()
    for col in col_lst:
        temcorr_df = mask_equal(df, col, "Noise")
        noise_df = pd.concat([noise_df, temcorr_df], sort=False)
        df = mask_unequal(df, col, "Noise")

    df = pd.concat([df, noise_df], sort=False)

    df = df.reset_index(drop=True)

    df[sig_col] = df[correct_p_col].map(label_sig)

    return df


def calc_corr_stat(df, x_col, y_col, return_df=False, use_kt=False):

    if len(df) < 2:
        pearson = np.nan
        p_val = np.nan
    else:
        df[x_col] = df[x_col].map(float)
        df[y_col] = df[y_col].map(float)

        x_lst = lst_col(df, x_col)
        y_lst = lst_col(df, y_col)

        if use_kt:
            corr, p_val = stats.kendalltau(x_lst, y_lst)
        else:
            corr, p_val = stats.pearsonr(x_lst, y_lst)

    if return_df:
        return pd.DataFrame({corr_col: [corr], p_col: [p_val]})
    else:
        return (corr, p_val)


def calc_corr(df, x_col, y_col, hue_cols=None, correct_method="fdr_bh", use_kt=False):

    val_df = df.copy(deep=True)

    if hue_cols is None:
        corr_df = calc_corr_stat(val_df, x_col, y_col, return_df=True, use_kt=use_kt)
    else:
        hue_col_lst = type_lst(hue_cols)

        corr_df = val_df.copy(deep=True)

        corr_df.index.name = index_col
        corr_df = corr_df.reset_index()

        corr_df = (
            corr_df.groupby(hue_col_lst)[index_col]
            .nunique()
            .reset_index(name=total_col)
        )

        df_index_lst = list(corr_df.index.values)

        for index in df_index_lst:

            hue_df = val_df.copy(deep=True)

            for col in hue_col_lst:

                val = corr_df.at[index, col]

                hue_df = mask_equal(hue_df, col, val)

            corr = calc_corr_stat(hue_df, x_col, y_col, use_kt=use_kt)

            corr_df.at[index, corr_col] = corr[0]
            corr_df.at[index, p_col] = corr[1]

        if correct_method is not None:
            corr_df = correct_p_vals(corr_df, correct_method=correct_method)

    corr_df = corr_df.sort_values(by=hue_col_lst)

    corr_df = corr_df.reset_index(drop=True)

    corr_df[sig_col] = corr_df[correct_p_col].map(label_sig)

    return corr_df



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import warnings

warnings.filterwarnings("ignore")

import pandas as pd
from tqdm import tqdm
import concurrent.futures

from ..functions import *


def get_index_dih(
    df, index, dih_dict, resid_dict, max_ca_dist=4, ext_mult=1, coord_path_col=None
):

    if coord_path_col is None:
        coord_path_col = core_path_col

    index_df = get_df_at_index(df, index)

    coord_path = index_df.at[index, coord_path_col]

    if coord_path in list(dih_dict.keys()):

        modelid = fix_val(index_df.at[index, modelid_col], return_int=True)
        chainid = index_df.at[index, chainid_col]

        try:
            dih_resid_lst = list(dih_dict[coord_path][str(modelid)][chainid].keys())
        except:
            dih_resid_lst = list(dih_dict[coord_path][int(modelid)][chainid].keys())

        index_df.at[index, complete_col] = True

        bb_resid_lst = list()
        bb_seq_lst = list()

        sc_seq_lst = list()
        sc_resid_lst = list()
        sc_len_lst = list()

        for dih_col in dih_col_lst:

            resid_lst = resid_dict[dih_col]

            if resid_lst is not None:

                add_resid_lst = build_add_resid_lst(
                    coord_path,
                    modelid,
                    chainid,
                    resid_lst,
                    dih_resid_lst,
                    max_ca_dist=max_ca_dist,
                    ext_mult=ext_mult,
                )

                val = 999.00
                resname = "-"
                for i, add_resid in enumerate(add_resid_lst):

                    if str(add_resid) in dih_resid_lst:

                        try:
                            val = dih_dict[coord_path][str(modelid)][chainid][
                                str(add_resid)
                            ][dih_col]
                            resname = resname_to_letter(
                                dih_dict[coord_path][str(modelid)][chainid][
                                    str(add_resid)
                                ][resname_col]
                            )
                        except:
                            val = dih_dict[coord_path][int(modelid)][chainid][
                                str(add_resid)
                            ][dih_col]
                            resname = resname_to_letter(
                                dih_dict[coord_path][int(modelid)][chainid][
                                    str(add_resid)
                                ][resname_col]
                            )

                    if dih_col == phi_col:
                        bb_resid_lst.append(add_resid)
                        bb_seq_lst.append(resname)

                    if dih_col in sc_col_lst:
                        sc_resid_lst.append(add_resid)
                        sc_seq_lst.append(resname)

                    val_col = get_val_col(dih_col, i + 1)

                    index_df.at[index, val_col] = float(val)

                    if val == 999.00:
                        index_df.at[index, complete_col] = False

                    val = 999.00
                    resname = "-"

                if dih_col in sc_col_lst:
                    sc_len_lst.append(len(sc_resid_lst))

        index_df.at[index, bb_len_col] = len(bb_resid_lst)
        index_df.at[index, sc_len_col] = lst_to_str(sc_len_lst)
        index_df.at[index, bb_resid_col] = lst_to_str(bb_resid_lst)
        index_df.at[index, sc_resid_col] = lst_to_str(sc_resid_lst)
        index_df.at[index, bb_seq_col] = lst_to_str(bb_seq_lst, join_txt="")
        index_df.at[index, sc_seq_col] = lst_to_str(sc_seq_lst, join_txt="")

    else:
        index_df = pd.DataFrame()

    return index_df


def get_rama_str(bb_vals_lst):

    rama_str = ""

    for bb_vals in bb_vals_lst:

        if 999.00 in bb_vals:
            rama_type = "-"
        else:
            rama_type = get_rama_type(bb_vals[0], bb_vals[1], bb_vals[2])

        rama_str += rama_type

    return rama_str


def get_rot_str(rot_val_lst):

    rot_str = ""

    for rot_val in rot_val_lst:

        rot_str += get_rot_type(rot_val)

    return rot_str


def add_bb_rama(df):

    bb_resid_lst = get_col_val_lst(df, phi_col)

    if len(bb_resid_lst) > 0:

        index_lst = list(df.index.values)

        for index in index_lst:

            max_index = int(float(df.at[index, bb_len_col]))

            if max_index > 1:
                resid_lst = bb_resid_lst[:max_index]
            else:
                resid_lst = bb_resid_lst

            bb_val_lst = list()

            for resid in resid_lst[:max_index]:

                bb_val_lst.append(
                    (
                        df.at[index, get_val_col(phi_col, resid)],
                        df.at[index, get_val_col(psi_col, resid)],
                        df.at[index, get_val_col(omega_col, resid)],
                    )
                )

            df.at[index, rama_col] = get_rama_str(bb_val_lst)

    return df


def add_sc_rot(df):

    for index in list(df.index.values):

        if sc_len_col in list(df.columns):

            sc_len_lst = str_to_lst(df.at[index, sc_len_col])

            sc_val_lst = list()
            i = 0

            for sc_col in sc_col_lst:

                sc_resid_lst = get_col_val_lst(df, sc_col)

                if len(sc_resid_lst) > 0:

                    max_index = int(float(sc_len_lst[i]))

                    if max_index > 1:
                        resid_lst = sc_resid_lst[:max_index]
                    else:
                        resid_lst = sc_resid_lst

                    for resid in resid_lst:

                        sc_val_lst.append(df.at[index, get_val_col(sc_col, resid)])

                    i += 1

            if len(sc_val_lst) > 0:

                df.at[index, rotamer_col] = get_rot_str(sc_val_lst)

    return df


def build_dih_table(
    df,
    dih_dict,
    dih_table_path=None,
    bb_resids=None,
    chi1_resids=None,
    chi2_resids=None,
    altchi1_resids=None,
    altchi2_resids=None,
    chi3_resids=None,
    chi4_resids=None,
    chi5_resids=None,
    max_ca_dist=4,
    ext_mult=1,
    coord_path_col=None,
    num_cpu=1,
):

    df_col_lst = list(df.columns)

    check_col_lst = [
        bb_resid_col,
        sc_resid_col,
        bb_seq_col,
        sc_seq_col,
        bb_len_col,
        sc_len_col,
        rama_col,
        rotamer_col,
    ]

    for check_col in check_col_lst:
        if check_col in df_col_lst:
            del df[check_col]

    for dih_col in dih_col_lst:
        for val_col in get_val_col_lst(df, dih_col):
            del df[val_col]

    angle_resids_lst = [
        bb_resids,
        bb_resids,
        bb_resids,
        chi1_resids,
        chi2_resids,
        altchi1_resids,
        altchi2_resids,
        chi3_resids,
        chi4_resids,
        chi5_resids,
    ]

    resid_dict = dict()

    for index, angle_resids in enumerate(angle_resids_lst):

        resid_dict[dih_col_lst[index]] = res_to_lst(angle_resids)

    dih_df = pd.DataFrame()

    if num_cpu == 1:
        for index in tqdm(
            list(df.index.values),
            desc="Building dihedral table",
            position=0,
            leave=True,
        ):
            index_df = get_index_dih(
                df,
                index,
                dih_dict,
                resid_dict,
                max_ca_dist=max_ca_dist,
                ext_mult=ext_mult,
                coord_path_col=coord_path_col,
            )
            for col in list(index_df.columns):
                if col not in list(dih_df.columns):
                    dih_df[col] = 999.00

            dih_df = pd.concat([dih_df, index_df], sort=False)
    else:
        with concurrent.futures.ProcessPoolExecutor(max_workers=num_cpu) as executor:
            job_lst = [
                executor.submit(
                    get_index_dih,
                    df,
                    index,
                    dih_dict,
                    resid_dict,
                    max_ca_dist=max_ca_dist,
                    ext_mult=ext_mult,
                    coord_path_col=coord_path_col,
                )
                for index in list(df.index.values)
            ]

            for job in tqdm(
                concurrent.futures.as_completed(job_lst),
                desc="Building dihedral table",
                total=len(job_lst),
                miniters=1,
                position=0,
                leave=True,
            ):
                index_df = job.result()

                for col in list(index_df.columns):
                    if col not in list(dih_df.columns):
                        dih_df[col] = 999.00

                dih_df = pd.concat([dih_df, index_df], sort=False)

    dih_df = dih_df.reset_index(drop=True)

    for dih_col in dih_col_lst:
        for val_col in get_val_col_lst(dih_df, dih_col):
            dih_df[val_col] = dih_df[val_col].fillna(999.00)

    dih_df = add_bb_rama(dih_df)
    dih_df = add_sc_rot(dih_df)

    print("Built dihedral table!")

    if dih_table_path is not None:
        save_table(dih_table_path, dih_df)
    else:
        return dih_df


from src.download.modules import *


def look_what_is_inside(format_to_look_at,
                        default_input_path_to_mmCIF=current_directory + "/mmCIF",
                        default_input_path_to_PDB=current_directory + "/PDB",
                        default_input_path_to_SIFTS=current_directory + "/SIFTS",
                        default_output_path_to_mmCIF=current_directory + "/output_mmCIF",
                        default_output_path_to_PDB=current_directory + "/output_PDB",
                        default_input_path_to_mmCIF_assembly=current_directory + "/mmCIF_assembly",
                        default_input_path_to_PDB_assembly=current_directory + "/PDB_assembly",
                        default_output_path_to_mmCIF_assembly=current_directory + "/output_mmCIF_assembly",
                        default_output_path_to_PDB_assembly=current_directory + "/output_PDB_assembly"):
    if format_to_look_at == "SIFTS":
        if not os.path.exists(default_input_path_to_SIFTS):
            os.makedirs(default_input_path_to_SIFTS)
        result = [f for f in listdir(default_input_path_to_SIFTS) if isfile(join(default_input_path_to_SIFTS, f))]
        return result
    if format_to_look_at == "mmCIF":
        if not os.path.exists(default_input_path_to_mmCIF):
            os.makedirs(default_input_path_to_mmCIF)
        result = [f for f in listdir(default_input_path_to_mmCIF) if isfile(join(default_input_path_to_mmCIF, f))]
        return result
    if format_to_look_at == "PDB":
        if not os.path.exists(default_input_path_to_PDB):
            os.makedirs(default_input_path_to_PDB)
        result = [f for f in listdir(default_input_path_to_PDB) if isfile(join(default_input_path_to_PDB, f))]
        return result
    if format_to_look_at == "output_mmCIF":
        if not os.path.exists(default_output_path_to_mmCIF):
            os.makedirs(default_output_path_to_mmCIF)
        result = [f for f in listdir(default_output_path_to_mmCIF) if isfile(join(default_output_path_to_mmCIF, f))]
        return result
    if format_to_look_at == "output_PDB":
        if not os.path.exists(default_output_path_to_PDB):
            os.makedirs(default_output_path_to_PDB)
        result = [f for f in listdir(default_output_path_to_PDB) if isfile(join(default_output_path_to_PDB, f))]
        return result
    if format_to_look_at == "mmCIF_assembly":
        if not os.path.exists(default_input_path_to_mmCIF_assembly):
            os.makedirs(default_input_path_to_mmCIF_assembly)
        result = [f for f in listdir(default_input_path_to_mmCIF_assembly) if isfile(join(default_input_path_to_mmCIF_assembly, f))]
        return result
    if format_to_look_at == "PDB_assembly":
        if not os.path.exists(default_input_path_to_PDB_assembly):
            os.makedirs(default_input_path_to_PDB_assembly)
        result = [f for f in listdir(default_input_path_to_PDB_assembly) if isfile(join(default_input_path_to_PDB_assembly, f))]
        return result
    if format_to_look_at == "output_mmCIF_assembly":
        if not os.path.exists(default_output_path_to_mmCIF_assembly):
            os.makedirs(default_output_path_to_mmCIF_assembly)
        result = [f for f in listdir(default_output_path_to_mmCIF_assembly) if isfile(join(default_output_path_to_mmCIF_assembly, f))]
        return result
    if format_to_look_at == "output_PDB_assembly":
        if not os.path.exists(default_output_path_to_PDB_assembly):
            os.makedirs(default_output_path_to_PDB_assembly)
        result = [f for f in listdir(default_output_path_to_PDB_assembly) if isfile(join(default_output_path_to_PDB_assembly, f))]
        return result


""" usage of look_what_is_inside()"""
# input_mmCIF_files_were_found = look_what_is_inside("mmCIF")
# input_PDB_files_were_found = look_what_is_inside("PDB")
# input_SIFTS_files_were_found = look_what_is_inside("SIFTS")
# output_PDB_files_were_found = look_what_is_inside("output_PDB")
# output_mmCIF_files_were_found = look_what_is_inside("output_mmCIF")

# print(len(input_mmCIF_files_were_found))
# print(len(input_PDB_files_were_found))
# print(len(input_SIFTS_files_were_found))
# print(len(output_mmCIF_files_were_found))
# print(len(output_PDB_files_were_found))



from src.download.modules import *
from src.download.downloadwithThreadPool import download_with_pool, url_formation_for_pool
from src.renum.shared.SIFTS_tree_parser import SIFTS_tree_parser


def try_SIFTS_tree_parser(default_input_path_to_SIFTS, SIFTS_name):
    product_tree_SIFTS = 0
    for _ in range(3):
        try:
            handle_SIFTS = gzip.open(Path(str(default_input_path_to_SIFTS) + "/" + SIFTS_name), 'rt')
            product_tree_SIFTS = SIFTS_tree_parser(handle_SIFTS)
            break
        except EOFError:
            os.remove(Path(str(default_input_path_to_SIFTS) + "/" + SIFTS_name))
            download_with_pool(url_formation_for_pool("SIFTS", [SIFTS_name])[0])
        except ValueError:
            os.remove(Path(str(default_input_path_to_SIFTS) + "/" + SIFTS_name))
            download_with_pool(url_formation_for_pool("SIFTS", [SIFTS_name])[0])
        except OSError:
            download_with_pool(url_formation_for_pool("SIFTS", [SIFTS_name])[0])
        except:
            download_with_pool(url_formation_for_pool("SIFTS", [SIFTS_name])[0])
    return product_tree_SIFTS



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import os
import argparse
import sys
import pyfiglet


from scripts import *
from scripts.functions import *


def main(args):

    classify = args.classify
    build = args.build
    gui = args.gui

    out = args.out
    cpu = args.cpu

    if classify is not None:
        classify_rascore(
            coord_paths=classify,
            out_path=out,
            num_cpu=cpu,
        )
    elif build is not None:
        pdbaa_fasta_path = None
        if build is not True:
            pdbaa_fasta_path = build
        prep_rascore(build_path=out)
        build_rascore(
            out_path=out,
            pdbaa_fasta_path=pdbaa_fasta_path,
            num_cpu=cpu,
        )
    elif gui is not None:
        if gui is not True:
            prep_rascore(build_path=gui)
            entry_table_path = get_file_path(entry_table_file, dir_path=gui)
            copy_path(
                entry_table_path,
                get_file_path(
                    entry_table_file,
                    get_dir_path(
                        dir_str=f"{scripts_str}/{data_str}",
                        dir_path=get_dir_name(__file__),
                    ),
                ),
            )

        rascore_app_path = get_file_path(
            "rascore_gui.py",
            dir_path=get_dir_name(__file__),
        )

        os.system(f"streamlit run {rascore_app_path}")


def cli(args=None):

    print(pyfiglet.figlet_format("rascore"))
    print("A tool for analyzing the conformations of RAS structures\n")
    print("Author: Mitchell Isaac Parker <mitch.isaac.parker@gmail.com>")
    print("License: MIT License\n")

    if not args:
        args = sys.argv[1:]

    parser = argparse.ArgumentParser(
        description="rascore: A tool for analyzing the conformations of RAS structures"
    )

    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        "-classify",
        "--classify",
        nargs="+",
        required=False,
        help="path to coordinate file(s) of RAS structures to conformationally classify provided as a space-separated list, line-sepated file, or tab-seperated file (output files saved to rascore_classify in current working directory unless an output directory path is specified)",
    )
    group.add_argument(
        "-build",
        "--build",
        type=str,
        const=True,
        nargs="?",
        required=False,
        help="build or update rascore database from the Protein Data Bank (output files saved to rascore_build in current working directory unless an output directory path is specified)",
    )
    group.add_argument(
        "-gui",
        "--gui",
        type=str,
        const=True,
        nargs="?",
        required=False,
        help="path to rascore database directory (can run limited version if not specified) for running the rascore GUI application (output files saved to rascore_app in current working directory unless an output directory path is specified)",
    )
    parser.add_argument(
        "-out",
        "--out",
        type=str,
        required=False,
        help=f"output directory path (default = {os.getcwd()}",
    )
    parser.add_argument(
        "-cpu",
        "--cpu",
        type=int,
        default=1,
        required=False,
        help="number of CPUs to use (default = 1)",
    )
    args = parser.parse_args(args)
    main(args)


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['rascore',
 'rascore.scripts',
 'rascore.scripts.PDBrenum',
 'rascore.scripts.PDBrenum.src.download',
 'rascore.scripts.PDBrenum.src.renum.PDB',
 'rascore.scripts.PDBrenum.src.renum.mmCIF',
 'rascore.scripts.PDBrenum.src.renum.shared',
 'rascore.scripts.constants',
 'rascore.scripts.functions',
 'rascore.scripts.pages',
 'rascore.scripts.tools']

package_data = \
{'': ['*'],
 'rascore': ['.streamlit/*'],
 'rascore.scripts': ['data/*',
                     'data/rascore_cluster/*',
                     'data/rascore_cluster/SW1/*',
                     'data/rascore_cluster/SW1/SW1_0P/*',
                     'data/rascore_cluster/SW1/SW1_2P/*',
                     'data/rascore_cluster/SW1/SW1_3P/*',
                     'data/rascore_cluster/SW2/*',
                     'data/rascore_cluster/SW2/SW2_0P/*',
                     'data/rascore_cluster/SW2/SW2_2P/*',
                     'data/rascore_cluster/SW2/SW2_3P/*'],
 'rascore.scripts.PDBrenum': ['src/*', 'src/renum/*']}

install_requires = \
['Bio>=1.3.3',
 'CairoSVG>=2.5.2',
 'lxml>=4.6.3',
 'matplotlib>=3.3.4',
 'matplotlib_venn>=0.11.6',
 'numpy>=1.20.1',
 'pandas>=1.2.4',
 'pendulum>=2.1.2',
 'pyfiglet>=0.8.post1',
 'rdkit-pypi>=2021.9.4',
 'requests>=2.25.1',
 'scipy>=1.6.2',
 'seaborn>=0.11.1',
 'statannot>=0.2.3',
 'statsmodels>=0.12.2',
 'streamlit>=1.4.0',
 'tqdm>=4.59.0']

entry_points = \
{'console_scripts': ['rascore = rascore.rascore_cli:cli']}

setup_kwargs = {
    'name': 'rascore',
    'version': '0.1.21',
    'description': 'A tool for analyzing the conformations of RAS structures',
    'long_description': None,
    'author': 'mitch-parker',
    'author_email': 'mitch.isaac.parker@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.10',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import streamlit as st
import py3Dmol
from stmol import showmol

from .file import entry_table_file
from .table import mask_equal
from .path import (
    get_dir_name,
    get_dir_path,
    get_file_path,
    load_table,
    pages_str,
    data_str,
)
from .col import *
from .lst import *


def load_st_table(file_path):

    dir_path = get_dir_name(file_path)
    dir_path = dir_path.split(pages_str)[0]
    dir_path += data_str

    return load_table(
        get_file_path(
            entry_table_file,
            dir_path=get_dir_path(dir_path=dir_path),
        )
    )


@st.cache
def mask_st_table(df, col_dict):

    mask_df = df.copy()

    for col in col_dict.keys():
        mask_df = mask_equal(mask_df, col, col_dict[col])

    return mask_df


def rename_st_cols(df, col_lst=None):

    if col_lst is None:
        col_lst = list(rename_col_dict.keys())

    return df.loc[:, col_lst].rename(columns=rename_col_dict)


def show_st_dataframe(df, st_col=None):

    hide_dataframe_row_index = """
            <style>
            .row_heading.level0 {display:none}
            .blank {display:none}
            </style>
            """

    st.markdown(hide_dataframe_row_index, unsafe_allow_html=True)

    if st_col is None:
        st.dataframe(df)
    else:
        st_col.dataframe(df)


def show_st_table(df, st_col=None):

    hide_table_row_index = """
            <style>
            tbody th {display:none}
            .blank {display:none}
            </style>
            """
    st.markdown(hide_table_row_index, unsafe_allow_html=True)

    if st_col is None:
        st.table(df)
    else:
        st_col.table(df)


def show_st_structure(
    pdb_code,
    style_lst=None,
    label_lst=None,
    zoom_dict=None,
    surface_lst=None,
    cartoon_style="trace",
    cartoon_radius=0.2,
    cartoon_color="lightgray",
    zoom=1,
    width=700,
    height=500,
):
    view = py3Dmol.view(query=f"pdb:{pdb_code}", width=width, height=height)

    view.setStyle(
        {
            "cartoon": {
                "style": cartoon_style,
                "color": cartoon_color,
                "thickness": cartoon_radius,
            }
        }
    )

    view.setViewStyle({"style": "outline", "color": "black", "width": 0.05})

    if surface_lst is not None:
        for surface in surface_lst:
            view.addSurface(py3Dmol.VDW, surface[0], surface[1])

    if style_lst is not None:
        for style in style_lst:
            view.addStyle(
                style[0],
                style[1],
            )

    if label_lst is not None:
        for label in label_lst:
            view.addLabel(label[0], label[1], label[2])

    if zoom_dict is None:
        view.zoomTo()
    else:
        view.zoomTo(zoom_dict)
    view.zoom(zoom)
    showmol(view, height=height, width=width)



#####################################################################################################################################################
# DEPENDENCIES #
#####################################################################################################################################################


# import os
# import argparse

from src.download.modules import *
from src.download.lefttorenumber import left_to_renumber_mmCIF, left_to_renumber_PDB
from src.download.inputtextfileparser import input_text_file_parser
from src.download.shortusagemessage import short_usage_messenger
from src.download.longusagemessage import long_usage_messenger
from src.download.supremedownloader import supreme_download_master
from src.download.lookfilesinside import look_what_is_inside
from src.download.downloadwithThreadPool import run_downloads_with_ThreadPool, url_formation_for_pool, download_pdb_assemblies_list_with_lxml

from src.renum.PDB.new_PDB import ProcessPool_run_renum_PDB
from src.renum.shared.write_log import log_writer
from src.renum.mmCIF.ProcessPool_run_renum_mmCIF import ProcessPool_run_renum_mmCIF
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

current_directory = os.getcwd()
exception_AccessionIDs = ["P42212", "Q17104", "Q27903", "Q93125", "P03069", "D3DLN9", "Q96UT3", "P0ABE7", "P00192",
                          "P76805", "Q8XCE3", "P00720", "Q38170", "Q94N07", "P0AEX9", "P02928", "Q2M6S0"]

# os.environ["HTTPS_PROXY"] = "http://155.247.166.25:8080"
# os.environ["HTTP_PROXY"] = "http://155.247.166.25:8080"
# os.environ["FTP_PROXY"] = "http://155.247.166.25:8080"
# os.environ["https_proxy"] = "http://155.247.166.25:8080"
# os.environ["http_proxy"] = "http://155.247.166.25:8080"
# os.environ["ftp_proxy"] = "http://155.247.166.25:8080"

# IMPORTANT!!! Check your network connection: Wi-Fi or Wired Ethernet

#####################################################################################################################################################
# ARGUMENTS #
#####################################################################################################################################################


argpar = argparse.ArgumentParser(usage=short_usage_messenger(), add_help=False)

argpar.add_argument("-h", action="store_true", help=argparse.SUPPRESS)
argpar.add_argument("--help", action="store_true", help=argparse.SUPPRESS)

argpar.add_argument("-rftf", "--renumber_from_text_file", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-rfla", "--renumber_from_list_of_arguments", metavar="6dbp 3v03 2jit", nargs="*", type=str, help=argparse.SUPPRESS)

argpar.add_argument("-dftf", "--download_from_text_file", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-dfla", "--download_from_list_of_arguments", metavar="6dbp 3v03 2jit", nargs="+", type=str, help=argparse.SUPPRESS)

argpar.add_argument("-redb", "--renumber_entire_database", action="store_true", help=argparse.SUPPRESS)
argpar.add_argument("-dall", "--download_entire_database", action="store_true", help=argparse.SUPPRESS)
argpar.add_argument("-refr", "--refresh_entire_database", action="store_true", help=argparse.SUPPRESS)

argpar.add_argument("-PDB", "--PDB_format_only", action="store_true", help=argparse.SUPPRESS)
argpar.add_argument("-mmCIF", "--mmCIF_format_only", action="store_true", help=argparse.SUPPRESS)
argpar.add_argument("-PDB_assembly", "--PDB_assembly_format_only", action="store_true", help=argparse.SUPPRESS)
argpar.add_argument("-mmCIF_assembly", "--mmCIF_assembly_format_only", action="store_true", help=argparse.SUPPRESS)
argpar.add_argument("-all", "--all_formats", action="store_true", help=argparse.SUPPRESS)

argpar.add_argument("-sipm", "--set_default_input_path_to_mmCIF", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-sipma", "--set_default_input_path_to_mmCIF_assembly", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-sipp", "--set_default_input_path_to_PDB", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-sippa", "--set_default_input_path_to_PDB_assembly", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-sips", "--set_default_input_path_to_SIFTS", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-sopm", "--set_default_output_path_to_mmCIF", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-sopma", "--set_default_output_path_to_mmCIF_assembly", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-sopp", "--set_default_output_path_to_PDB", type=str, help=argparse.SUPPRESS)
argpar.add_argument("-soppa", "--set_default_output_path_to_PDB_assembly", type=str, help=argparse.SUPPRESS)

argpar.add_argument("-sdmn", "--set_default_mmCIF_num", type=int, help=argparse.SUPPRESS)
argpar.add_argument("-sdpn", "--set_default_PDB_num", type=int, help=argparse.SUPPRESS)

argpar.add_argument("-nproc", "--set_number_of_processes", type=int, help=argparse.SUPPRESS)
argpar.add_argument("-offz", "--set_to_off_mode_gzip", action="store_true", help=argparse.SUPPRESS)


args = argpar.parse_args()


#####################################################################################################################################################
# FLAGS #
#####################################################################################################################################################


if args.help:
    print(long_usage_messenger())

if args.h:
    print(short_usage_messenger())

if args.set_default_input_path_to_mmCIF:
    default_input_path_to_mmCIF = args.set_default_input_path_to_mmCIF
else:
    default_input_path_to_mmCIF = current_directory + "/mmCIF"

if args.set_default_input_path_to_mmCIF_assembly:
    default_input_path_to_mmCIF_assembly = args.set_default_input_path_to_mmCIF_assembly
else:
    default_input_path_to_mmCIF_assembly = current_directory + "/mmCIF_assembly"

if args.set_default_input_path_to_PDB:
    default_input_path_to_PDB = args.set_default_input_path_to_PDB
else:
    default_input_path_to_PDB = current_directory + "/PDB"

if args.set_default_input_path_to_PDB_assembly:
    default_input_path_to_PDB_assembly = args.set_default_input_path_to_PDB_assembly
else:
    default_input_path_to_PDB_assembly = current_directory + "/PDB_assembly"

if args.set_default_input_path_to_SIFTS:
    default_input_path_to_SIFTS = args.set_default_input_path_to_SIFTS
else:
    default_input_path_to_SIFTS = current_directory + "/SIFTS"

if args.set_default_output_path_to_mmCIF:
    default_output_path_to_mmCIF = args.set_default_output_path_to_mmCIF
else:
    default_output_path_to_mmCIF = current_directory + "/output_mmCIF"

if args.set_default_output_path_to_mmCIF_assembly:
    default_output_path_to_mmCIF_assembly = args.set_default_output_path_to_mmCIF_assembly
else:
    default_output_path_to_mmCIF_assembly = current_directory + "/output_mmCIF_assembly"

if args.set_default_output_path_to_PDB:
    default_output_path_to_PDB = args.set_default_output_path_to_PDB
else:
    default_output_path_to_PDB = current_directory + "/output_PDB"

if args.set_default_output_path_to_PDB_assembly:
    default_output_path_to_PDB_assembly = args.set_default_output_path_to_PDB_assembly
else:
    default_output_path_to_PDB_assembly = current_directory + "/output_PDB_assembly"

if args.set_default_mmCIF_num:
    default_mmCIF_num = args.set_default_mmCIF_num
else:
    default_mmCIF_num = 50000

if args.set_default_PDB_num:
    default_PDB_num = args.set_default_PDB_num
else:
    default_PDB_num = 5000

if args.set_to_off_mode_gzip:
    gzip_mode = "off"
else:
    gzip_mode = "on"

if args.set_number_of_processes:
    nproc = args.set_number_of_processes
else:
    nproc = None


if __name__ == "__main__":

    #################################################################################################################################################
    # PARTIAL DB WORK #
    #################################################################################################################################################

    # RENUMBER
    # RENUMBER FROM TEXT FILE or RENUMBER FROM LIST OF ARGUMENTS
    if args.renumber_from_text_file or args.renumber_from_list_of_arguments:
        if args.renumber_from_text_file:
            parsed_input_text = (input_text_file_parser(args.renumber_from_text_file))
        else:
            parsed_input_text = args.renumber_from_list_of_arguments

        if args.all_formats:
            urls_to_target_mmCIF_files = url_formation_for_pool("mmCIF", parsed_input_text, default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            urls_to_target_mmCIF_assembly_files = url_formation_for_pool("mmCIF_assembly", parsed_input_text,
                                                                         default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly)
            urls_to_target_PDB_files = url_formation_for_pool("PDB", parsed_input_text, default_input_path_to_PDB=default_input_path_to_PDB)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

            run_downloads_with_ThreadPool("mmCIF", urls_to_target_mmCIF_files, default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            run_downloads_with_ThreadPool("mmCIF_assembly", urls_to_target_mmCIF_assembly_files,
                                          default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly)
            run_downloads_with_ThreadPool("PDB", urls_to_target_PDB_files, default_input_path_to_PDB=default_input_path_to_PDB)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

            # renum PDB
            passed_as_arg_file_4Char_PDB = list()
            for file_name in parsed_input_text:
                passed_as_arg_file_4Char_PDB.append(file_name[:4])
            input_PDB_files_were_found = look_what_is_inside("PDB", default_input_path_to_PDB=default_input_path_to_PDB)
            target_files_list_PDB = list()
            for file_name in input_PDB_files_were_found:
                if file_name[3:7] in passed_as_arg_file_4Char_PDB:
                    target_files_list_PDB.append(file_name)
            ProcessPool_run_renum_PDB("PDB", target_files_list_PDB, default_input_path_to_PDB, default_input_path_to_SIFTS,
                                      default_output_path_to_PDB, default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)

            # renum mmCIF_assembly
            input_mmCIF_files_were_found = look_what_is_inside("mmCIF_assembly",
                                                               default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly)
            passed_as_arg_file_4Char_mmCIF = list()
            for file_name in parsed_input_text:
                passed_as_arg_file_4Char_mmCIF.append(file_name[:4])
            target_files_list_mmCIF = list()
            for file_name in input_mmCIF_files_were_found:
                if file_name[:4] in passed_as_arg_file_4Char_mmCIF:
                    target_files_list_mmCIF.append(file_name)

            ProcessPool_run_renum_mmCIF("mmCIF_assembly", target_files_list_mmCIF, default_input_path_to_mmCIF_assembly,
                                        default_input_path_to_SIFTS, default_output_path_to_mmCIF_assembly, default_mmCIF_num,
                                        gzip_mode, exception_AccessionIDs, nproc)

            # renum mmCIF
            input_mmCIF_files_were_found = look_what_is_inside("mmCIF", default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            passed_as_arg_file_4Char_mmCIF = list()
            for file_name in parsed_input_text:
                passed_as_arg_file_4Char_mmCIF.append(file_name[:4])
            target_files_list_mmCIF = list()
            for file_name in input_mmCIF_files_were_found:
                if file_name[:4] in passed_as_arg_file_4Char_mmCIF:
                    target_files_list_mmCIF.append(file_name)
            res = ProcessPool_run_renum_mmCIF("mmCIF", target_files_list_mmCIF, default_input_path_to_mmCIF, default_input_path_to_SIFTS,
                                              default_output_path_to_mmCIF, default_mmCIF_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

            # renum PDB_assembly
            if not os.path.exists(default_input_path_to_PDB_assembly):
                os.makedirs(default_input_path_to_PDB_assembly)
            urls_to_target_PDB_files = list()
            all_urls_set = download_pdb_assemblies_list_with_lxml()
            for url in all_urls_set:
                for PDB_id in parsed_input_text:
                    if PDB_id in url:
                        urls_to_target_PDB_files.append(url)
            run_downloads_with_ThreadPool("PDB_assembly", urls_to_target_PDB_files,
                                          default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly)
            input_PDB_files_were_found = look_what_is_inside("PDB_assembly", default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly)

            passed_as_arg_file_4Char_PDB = list()
            for file_name in parsed_input_text:
                passed_as_arg_file_4Char_PDB.append(file_name[:4])
            target_files_list_PDB = list()
            for file_name in input_PDB_files_were_found:
                if file_name[:4] in passed_as_arg_file_4Char_PDB:
                    target_files_list_PDB.append(file_name)
            ProcessPool_run_renum_PDB("PDB_assembly", target_files_list_PDB, default_input_path_to_PDB_assembly, default_input_path_to_SIFTS,
                                      default_output_path_to_PDB_assembly, default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)

        elif args.PDB_assembly_format_only:
            if not os.path.exists(default_input_path_to_PDB_assembly):
                os.makedirs(default_input_path_to_PDB_assembly)
            urls_to_target_PDB_files = list()
            all_urls_set = download_pdb_assemblies_list_with_lxml()
            for url in all_urls_set:
                for PDB_id in parsed_input_text:
                    if PDB_id in url:
                        urls_to_target_PDB_files.append(url)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("PDB_assembly", urls_to_target_PDB_files,
                                          default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

            input_PDB_files_were_found = look_what_is_inside("PDB_assembly", default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly)
            passed_as_arg_file_4Char_PDB = list()
            for file_name in parsed_input_text:
                passed_as_arg_file_4Char_PDB.append(file_name[:4])
            target_files_list_PDB = list()
            for file_name in input_PDB_files_were_found:
                if file_name[:4] in passed_as_arg_file_4Char_PDB:
                    target_files_list_PDB.append(file_name)

            res = ProcessPool_run_renum_PDB("PDB_assembly", target_files_list_PDB, default_input_path_to_PDB_assembly, default_input_path_to_SIFTS,
                                            default_output_path_to_PDB_assembly, default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        elif args.PDB_format_only:
            urls_to_target_PDB_files = url_formation_for_pool("PDB", parsed_input_text, default_input_path_to_PDB=default_input_path_to_PDB)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("PDB", urls_to_target_PDB_files, default_input_path_to_PDB=default_input_path_to_PDB)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

            input_PDB_files_were_found = look_what_is_inside("PDB", default_input_path_to_PDB=default_input_path_to_PDB)
            passed_as_arg_file_4Char_PDB = list()
            for file_name in parsed_input_text:
                passed_as_arg_file_4Char_PDB.append(file_name[:4])
            target_files_list_PDB = list()
            for file_name in input_PDB_files_were_found:
                if file_name[3:7] in passed_as_arg_file_4Char_PDB:
                    target_files_list_PDB.append(file_name)

            res = ProcessPool_run_renum_PDB("PDB", target_files_list_PDB, default_input_path_to_PDB, default_input_path_to_SIFTS,
                                            default_output_path_to_PDB, default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        elif args.mmCIF_assembly_format_only:
            urls_to_target_mmCIF_files = url_formation_for_pool("mmCIF_assembly", parsed_input_text,
                                                                default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("mmCIF_assembly", urls_to_target_mmCIF_files,
                                          default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

            input_mmCIF_files_were_found = look_what_is_inside("mmCIF_assembly",
                                                               default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly)
            passed_as_args_files_list_4Char = list()
            for file_name in parsed_input_text:
                passed_as_args_files_list_4Char.append(file_name[:4])

            target_files_list = list()
            for file_name in input_mmCIF_files_were_found:
                if file_name[:4] in passed_as_args_files_list_4Char:
                    target_files_list.append(file_name)

            if not os.path.exists(default_output_path_to_mmCIF_assembly):
                os.makedirs(default_output_path_to_mmCIF_assembly)

            res = ProcessPool_run_renum_mmCIF("mmCIF_assembly", target_files_list, default_input_path_to_mmCIF_assembly,
                                              default_input_path_to_SIFTS, default_output_path_to_mmCIF_assembly, default_mmCIF_num,
                                              gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        else:
            urls_to_target_mmCIF_files = url_formation_for_pool("mmCIF", parsed_input_text, default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("mmCIF", urls_to_target_mmCIF_files, default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

            input_mmCIF_files_were_found = look_what_is_inside("mmCIF", default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            passed_as_args_files_list_4Char = list()
            for file_name in parsed_input_text:
                passed_as_args_files_list_4Char.append(file_name[:4])

            target_files_list = list()
            for file_name in input_mmCIF_files_were_found:
                if file_name[:4] in passed_as_args_files_list_4Char:
                    target_files_list.append(file_name)

            if not os.path.exists(default_output_path_to_mmCIF):
                os.makedirs(default_output_path_to_mmCIF)
            res = ProcessPool_run_renum_mmCIF("mmCIF", target_files_list, default_input_path_to_mmCIF, default_input_path_to_SIFTS,
                                              default_output_path_to_mmCIF, default_mmCIF_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

    # DOWNLOAD
    # DOWNLOAD FROM TEXT FILE or DOWNLOAD FROM LIST
    if args.download_from_text_file or args.download_from_list_of_arguments:
        if args.download_from_text_file:
            parsed_input_text = (input_text_file_parser(args.download_from_text_file))
        else:
            parsed_input_text = args.download_from_list_of_arguments

        if args.all_formats:
            urls_to_target_mmCIF_files = url_formation_for_pool("mmCIF", parsed_input_text, default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            urls_to_target_PDB_files = url_formation_for_pool("PDB", parsed_input_text, default_input_path_to_PDB=default_input_path_to_PDB)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("mmCIF", urls_to_target_mmCIF_files, default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            run_downloads_with_ThreadPool("PDB", urls_to_target_PDB_files, default_input_path_to_PDB=default_input_path_to_PDB)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("mmCIF_assembly", urls_to_target_mmCIF_files,
                                          default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly)
            # PDB_assembly download
            urls_to_target_PDB_files = list()
            all_urls_set = download_pdb_assemblies_list_with_lxml()
            for url in all_urls_set:
                for PDB_id in parsed_input_text:
                    if PDB_id in url:
                        urls_to_target_PDB_files.append(url)
            run_downloads_with_ThreadPool("PDB_assembly", urls_to_target_PDB_files,
                                          default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly)

        elif args.PDB_assembly_format_only:
            urls_to_target_PDB_files = list()
            all_urls_set = download_pdb_assemblies_list_with_lxml()
            for url in all_urls_set:
                for PDB_id in parsed_input_text:
                    if PDB_id in url:
                        urls_to_target_PDB_files.append(url)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("PDB_assembly", urls_to_target_PDB_files,
                                          default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

        elif args.PDB_format_only:
            urls_to_target_PDB_files = url_formation_for_pool("PDB", parsed_input_text, default_input_path_to_PDB=default_input_path_to_PDB)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("PDB", urls_to_target_PDB_files, default_input_path_to_PDB=default_input_path_to_PDB)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

        elif args.mmCIF_assembly_format_only:
            urls_to_target_mmCIF_files = url_formation_for_pool("mmCIF_assembly", parsed_input_text,
                                                                default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("mmCIF_assembly", urls_to_target_mmCIF_files,
                                          default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

        else:
            urls_to_target_mmCIF_files = url_formation_for_pool("mmCIF", parsed_input_text, default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            urls_to_target_SIFTS_files = url_formation_for_pool("SIFTS", parsed_input_text, default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            run_downloads_with_ThreadPool("mmCIF", urls_to_target_mmCIF_files, default_input_path_to_mmCIF=default_input_path_to_mmCIF)
            run_downloads_with_ThreadPool("SIFTS", urls_to_target_SIFTS_files, default_input_path_to_SIFTS=default_input_path_to_SIFTS)

    #################################################################################################################################################
    # WHOLE DB WORK #
    #################################################################################################################################################

    # RENUMBER ENTIRE DB
    if args.renumber_entire_database:
        if args.all_formats:
            print("Starting to renumber all databases...")
            print("Please, be patient...")
            # renumber mmCIF
            supreme_download_master("mmCIF", default_input_path_to_mmCIF=default_input_path_to_mmCIF,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            mmCIF_files_left_to_renumber = left_to_renumber_mmCIF(default_input_path_to_mmCIF=default_input_path_to_mmCIF,
                                                                  default_output_path_to_mmCIF=default_output_path_to_mmCIF)
            res = ProcessPool_run_renum_mmCIF("mmCIF", mmCIF_files_left_to_renumber, default_input_path_to_mmCIF, default_input_path_to_SIFTS,
                                              default_output_path_to_mmCIF, default_mmCIF_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

            # renumber mmCIF_assembly
            supreme_download_master("mmCIF_assembly", default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            mmCIF_assembly_left_to_renumber = left_to_renumber_mmCIF(default_input_path_to_mmCIF=default_input_path_to_mmCIF_assembly,
                                                                     default_output_path_to_mmCIF=default_output_path_to_mmCIF_assembly)
            ProcessPool_run_renum_mmCIF("mmCIF_assembly", mmCIF_assembly_left_to_renumber, default_input_path_to_mmCIF_assembly,
                                        default_input_path_to_SIFTS, default_output_path_to_mmCIF_assembly, default_mmCIF_num,
                                        gzip_mode, exception_AccessionIDs, nproc)

            # renumber PDB_assembly
            supreme_download_master("PDB_assembly", default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            PDB_assembly_files_left_to_renumber = left_to_renumber_mmCIF(default_input_path_to_mmCIF=default_input_path_to_PDB_assembly,
                                                                         default_output_path_to_mmCIF=default_output_path_to_PDB_assembly)
            ProcessPool_run_renum_PDB("PDB_assembly", PDB_assembly_files_left_to_renumber, default_input_path_to_PDB_assembly,
                                      default_input_path_to_SIFTS, default_output_path_to_PDB_assembly, default_PDB_num,
                                      gzip_mode, exception_AccessionIDs, nproc)

            # renumber PDB
            supreme_download_master("PDB", default_input_path_to_PDB=default_input_path_to_PDB,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            PDB_files_left_to_renumber = left_to_renumber_PDB(default_input_path_to_PDB=default_input_path_to_PDB,
                                                              default_output_path_to_PDB=default_output_path_to_PDB)
            ProcessPool_run_renum_PDB("PDB", PDB_files_left_to_renumber, default_input_path_to_PDB, default_input_path_to_SIFTS,
                                      default_output_path_to_PDB, default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)

        elif args.PDB_assembly_format_only:
            print("Starting to renumber entire PDB_assembly database...")
            print("Please, be patient...")
            supreme_download_master("PDB_assembly", default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            PDB_assembly_files_left_to_renumber = left_to_renumber_mmCIF(default_input_path_to_mmCIF=default_input_path_to_PDB_assembly,
                                                                         default_output_path_to_mmCIF=default_output_path_to_PDB_assembly)
            res = ProcessPool_run_renum_PDB("PDB_assembly", PDB_assembly_files_left_to_renumber, default_input_path_to_PDB_assembly,
                                            default_input_path_to_SIFTS, default_output_path_to_PDB_assembly, default_PDB_num,
                                            gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        elif args.PDB_format_only:
            print("Starting to renumber entire PDB database...")
            print("Please, be patient...")
            supreme_download_master("PDB", default_input_path_to_PDB=default_input_path_to_PDB,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            PDB_files_left_to_renumber = left_to_renumber_PDB(default_input_path_to_PDB=default_input_path_to_PDB,
                                                              default_output_path_to_PDB=default_output_path_to_PDB)
            res = ProcessPool_run_renum_PDB("PDB", PDB_files_left_to_renumber, default_input_path_to_PDB, default_input_path_to_SIFTS,
                                            default_output_path_to_PDB, default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        elif args.mmCIF_assembly_format_only:
            print("Starting to renumber entire mmCIF_assembly database...")
            print("Please, be patient...")
            supreme_download_master("mmCIF_assembly", default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            mmCIF_assembly_left_to_renumber = left_to_renumber_mmCIF(default_input_path_to_mmCIF=default_input_path_to_mmCIF_assembly,
                                                                     default_output_path_to_mmCIF=default_output_path_to_mmCIF_assembly)
            res = ProcessPool_run_renum_mmCIF("mmCIF_assembly", mmCIF_assembly_left_to_renumber, default_input_path_to_mmCIF_assembly,
                                              default_input_path_to_SIFTS, default_output_path_to_mmCIF_assembly, default_mmCIF_num,
                                              gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        else:
            print("Starting to renumber entire mmCIF database...")
            print("Please, be patient...")
            supreme_download_master("mmCIF", default_input_path_to_mmCIF=default_input_path_to_mmCIF,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            mmCIF_files_left_to_renumber = left_to_renumber_mmCIF(default_input_path_to_mmCIF=default_input_path_to_mmCIF,
                                                                  default_output_path_to_mmCIF=default_output_path_to_mmCIF)
            res = ProcessPool_run_renum_mmCIF("mmCIF", mmCIF_files_left_to_renumber, default_input_path_to_mmCIF, default_input_path_to_SIFTS,
                                              default_output_path_to_mmCIF, default_mmCIF_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

    # DOWNLOAD ENTIRE DB
    if args.download_entire_database:
        if args.all_formats:
            print("Starting to download all databases...")
            print("Please, be patient...")
            supreme_download_master("all", default_input_path_to_mmCIF=default_input_path_to_mmCIF,
                                    default_input_path_to_PDB=default_input_path_to_PDB,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS,
                                    default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly,
                                    default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly)

        elif args.PDB_format_only:
            print("Starting to download entire PDB database...")
            print("Please, be patient...")
            supreme_download_master("PDB", default_input_path_to_PDB=default_input_path_to_PDB,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)

        elif args.PDB_assembly_format_only:
            print("Starting to download entire PDB_assembly database...")
            print("Please, be patient...")
            supreme_download_master("PDB_assembly", default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)

        elif args.mmCIF_assembly_format_only:
            print("Starting to download entire mmCIF_assembly database...")
            print("Please, be patient...")
            supreme_download_master("mmCIF_assembly", default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)

        else:
            print("Starting to download entire mmCIF database...")
            print("Please, be patient...")
            supreme_download_master("mmCIF", default_input_path_to_mmCIF=default_input_path_to_mmCIF,
                                    default_input_path_to_SIFTS=default_input_path_to_SIFTS)

    # REFRESH ENTIRE DB
    if args.refresh_entire_database:
        if args.all_formats:
            print("Starting to refresh all databases...")
            print("Please, be patient...")
            left_to_refresh = supreme_download_master("all", "refresh", default_input_path_to_mmCIF=default_input_path_to_mmCIF,
                                                      default_input_path_to_PDB=default_input_path_to_PDB,
                                                      default_input_path_to_SIFTS=default_input_path_to_SIFTS,
                                                      default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly,
                                                      default_input_path_to_PDB_assembly=default_input_path_to_mmCIF_assembly)
            left_to_refresh_mmCIF = left_to_refresh[0]
            left_to_refresh_PDB = left_to_refresh[1]
            lefttodownload_mmCIF_assemblies = left_to_refresh[2]
            lefttodownload_PDB_assemblies = left_to_refresh[3]
            ProcessPool_run_renum_PDB("PDB_assembly", lefttodownload_PDB_assemblies, default_input_path_to_PDB_assembly,
                                      default_input_path_to_SIFTS, default_output_path_to_PDB_assembly,
                                      default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)
            ProcessPool_run_renum_PDB("PDB", left_to_refresh_PDB, default_input_path_to_PDB, default_input_path_to_SIFTS, default_output_path_to_PDB,
                                      default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)

            ProcessPool_run_renum_mmCIF("mmCIF_assembly", lefttodownload_mmCIF_assemblies, default_input_path_to_mmCIF_assembly,
                                        default_input_path_to_SIFTS, default_output_path_to_mmCIF_assembly, default_mmCIF_num,
                                        gzip_mode, exception_AccessionIDs, nproc)

            res = ProcessPool_run_renum_mmCIF("mmCIF", left_to_refresh_mmCIF, default_input_path_to_mmCIF, default_input_path_to_SIFTS,
                                              default_output_path_to_mmCIF, default_mmCIF_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        elif args.PDB_assembly_format_only:
            print("Starting to refresh entire PDB_assembly database...")
            print("Please, be patient...")
            left_to_refresh_PDB_assembly = supreme_download_master("PDB_assembly", "refresh",
                                                                   default_input_path_to_PDB_assembly=default_input_path_to_PDB_assembly,
                                                                   default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            res = ProcessPool_run_renum_PDB("PDB_assembly", left_to_refresh_PDB_assembly, default_input_path_to_PDB_assembly,
                                            default_input_path_to_SIFTS, default_output_path_to_PDB_assembly,
                                            default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        elif args.PDB_format_only:
            print("Starting to refresh entire PDB database...")
            print("Please, be patient...")
            left_to_refresh_PDB = supreme_download_master("PDB", "refresh", default_input_path_to_PDB=default_input_path_to_PDB,
                                                          default_input_path_to_SIFTS=default_input_path_to_SIFTS)
            res = ProcessPool_run_renum_PDB("PDB", left_to_refresh_PDB, default_input_path_to_PDB, default_input_path_to_SIFTS,
                                            default_output_path_to_PDB, default_PDB_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        elif args.mmCIF_assembly_format_only:
            print("Starting to refresh entire mmCIF_assembly database...")
            print("Please, be patient...")
            left_to_refresh_mmCIF_assembly = supreme_download_master("mmCIF_assembly", "refresh",
                                                                     default_input_path_to_mmCIF_assembly=default_input_path_to_mmCIF_assembly,
                                                                     default_input_path_to_SIFTS=default_input_path_to_SIFTS)

            res = ProcessPool_run_renum_mmCIF("mmCIF_assembly", left_to_refresh_mmCIF_assembly, default_input_path_to_mmCIF_assembly,
                                              default_input_path_to_SIFTS, default_output_path_to_mmCIF_assembly, default_mmCIF_num,
                                              gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)

        else:
            print("Starting to refresh entire mmCIF database...")
            print("Please, be patient...")
            left_to_refresh_mmCIF = supreme_download_master("mmCIF", "refresh", default_input_path_to_mmCIF=default_input_path_to_mmCIF,
                                                            default_input_path_to_SIFTS=default_input_path_to_SIFTS)

            res = ProcessPool_run_renum_mmCIF("mmCIF", left_to_refresh_mmCIF, default_input_path_to_mmCIF, default_input_path_to_SIFTS,
                                              default_output_path_to_mmCIF, default_mmCIF_num, gzip_mode, exception_AccessionIDs, nproc)
            log_writer(res)



def renumbered_count_in_chains(chains_to_change_one_to_end, df_PDBe_PDB_UniProt_without_null_index_PDBe, mmCIF_name,
                               UniProt_conversion_dict, longest_AccessionID_list):
    nothing_changed = 1
    chain_total_renum = list()
    renum_for_all_chains = 0
    total_renum5000 = 0
    chains_to_change = sorted(chains_to_change_one_to_end)
    chain_PDBe_PDB = dict()
    # prot_len = len(df_PDBe_PDB_UniProt_without_null_index_PDBe["Three_Rows_CIF_Num_Uni"])
    # UniProt_total_renum = list()

    for chain in chains_to_change:
        total_count_per_chain = 0
        renum_for_the_chains = 0
        renum5000 = 0
        UniProt_set = set()

        for PDBe_num_Uni_PDB in df_PDBe_PDB_UniProt_without_null_index_PDBe["Three_Rows_CIF_Num_Uni"]:
            if chain == PDBe_num_Uni_PDB[2][2]:
                chain_PDBe_PDB[chain] = PDBe_num_Uni_PDB[3][2]
                if type(PDBe_num_Uni_PDB[4]) != float:
                    UniProt_set.add(PDBe_num_Uni_PDB[4])
                total_count_per_chain += 1
                if int(PDBe_num_Uni_PDB[1]) > 5000:
                    renum5000 += 1
                    total_renum5000 += 1
                elif PDBe_num_Uni_PDB[1] != PDBe_num_Uni_PDB[3][0]:
                    renum_for_all_chains += 1
                    renum_for_the_chains += 1

        for accession in UniProt_set:
            renum_for_accession = 0
            count_accession_len = 0
            for PDBe_num_Uni_PDB in df_PDBe_PDB_UniProt_without_null_index_PDBe["Three_Rows_CIF_Num_Uni"]:
                if accession == PDBe_num_Uni_PDB[4]:
                    if chain == PDBe_num_Uni_PDB[2][2]:
                        count_accession_len += 1
                if chain == PDBe_num_Uni_PDB[2][2] and accession == PDBe_num_Uni_PDB[4]:
                    if PDBe_num_Uni_PDB[1] != PDBe_num_Uni_PDB[3][0]:
                        renum_for_accession += 1

            if len(longest_AccessionID_list) != 0:
                if accession in longest_AccessionID_list:
                    AccessionID_human_read_longest = UniProt_conversion_dict.get(accession)
                    chain_total_renum.append(
                        [mmCIF_name[:4] + "*", chain, chain_PDBe_PDB[chain], accession, AccessionID_human_read_longest, count_accession_len,
                         total_count_per_chain, renum_for_accession, renum5000])
                else:
                    AccessionID_human_read = UniProt_conversion_dict.get(accession)
                    chain_total_renum.append(
                        [mmCIF_name[:4], chain, chain_PDBe_PDB[chain], accession, AccessionID_human_read, count_accession_len, total_count_per_chain,
                         renum_for_accession, renum5000])
            else:
                if type(accession) != float:
                    AccessionID_human_read = UniProt_conversion_dict.get(accession)
                    chain_total_renum.append(
                        [mmCIF_name[:4], chain, chain_PDBe_PDB[chain], accession, AccessionID_human_read, count_accession_len, total_count_per_chain,
                         renum_for_accession, renum5000])

    if renum_for_all_chains == 0 and total_renum5000 == 0:
        nothing_changed = 0

    return [chain_total_renum, nothing_changed]
